import {Noise} from "noisejs";
import { BC_CHUNKS_SETTINGS, PRNG } from "../../GlobalVariables/GlobalVariables";
import { clampNumber } from "../../Utils/Math.utils";
import { DWFBMSimplex2, FBMSimplex2 } from "../../Utils/Noise.utils";

/**
 * Terrain generator
 * 
 * cNoise 1:100 - continent type
 * mNoise 1:20 - mountains
 */

let cNoise = new Noise(Math.floor(PRNG() * 999999999999));
let tNoise = new Noise(Math.floor(PRNG() * 999999999999));

export function genTerrain(xMin, xMax, yMin, yMax)
{   
    /**
     * @type {Array<Number>}
     */
    let terrainResult = new Array(BC_CHUNKS_SETTINGS.width * BC_CHUNKS_SETTINGS.height);
    let mainValue = 0;
    let mountainsValue = 0;
    let temperatureValue = 0;

    let ii = 0;
    let jj = 0;
    for (let i = xMin; i < xMax; i++) {
        jj = 0;
        for (let j = yMin; j < yMax; j++) {
            mainValue = DWFBMSimplex2(cNoise, i / 1000.0, j / 1000.0, 3.0);
            mainValue *= mainValue;
            temperatureValue = DWFBMSimplex2(tNoise, i / 1000.0, j / 1000.0, 3.0);
            // mainValue = Math.abs(temperatureValue - mainValue);
            // if(temperatureValue > 0.5)
            // {
            mainValue *= DWFBMSimplex2(cNoise, i / 100, j / 100, 3.0);
            // }
            // mountainsValue = _normalizeNoiseValue(mNoise.simplex2(i / 200, j / 200));
            // mainValue += clampNumber(Math.pow(mountainsValue * 2, 5), 0.0, 1.0);
            // mainValue = clampNumber(mainValue, 0.0, 0.99);
            terrainResult[ii * BC_CHUNKS_SETTINGS.width + jj] = mainValue;
            // //soil
            // if(continentValue > 0.5)
            // {
            //     terrainResult[ii * BC_CHUNKS_SETTINGS.width + jj] = continentValue;
            // }
            // else //water
            // {
            //     terrainResult[ii * BC_CHUNKS_SETTINGS.width + jj] = continentValue * 0.3;
            // }
            jj++;
        }
        ii++;
    }
    return terrainResult;
}